@@ -1,4 +1,4 @@
-# Copyright (c) 2005-2007 Gentoo Foundation
+# Copyright (c) 2005-2023 Gentoo Authors
 # Copyright (c) 2007-2008 Roy Marples <roy@marples.name>
 # Released under the 2-clause BSD license.
 # shellcheck shell=sh disable=SC1008
@@ -21,29 +21,27 @@ requote()
 	printf "'%s' " "$@"
 }
 
+pppd_version_int() {
+	# 002004008 is v2.4.8
+	printf '%03d' $(/usr/sbin/pppd --version | awk '/pppd version/ {print $3}' | tr '.' ' ')
+}
+
 pppd_is_ge_248()
 {
-	local ver_str="$(/usr/sbin/pppd --version 2>&1 | grep -o '[[:digit:]\.]\+')"
-	local maj_ver="$(echo ${ver_str} | cut -d . -f 1)"
-	local min_ver="$(echo ${ver_str} | cut -d . -f 2)"
-	local patch_ver="$(echo ${ver_str} | cut -d . -f 3)"
+	[ $(pppd_version_int) -ge 002004008 ]
+}
 
-	if [ "${maj_ver}" -gt 2 ] ; then
-		return 0
-	elif [ "${maj_ver}" -eq 2 ] ; then
-		if [ "${min_ver}" -gt 4 ] ; then
-			return 0
-		elif [ "${min_ver}" -eq 4 ] ; then
-			if [ "${patch_ver}" -ge 8 ] ; then
-				return 0
-			else
-				return 1
-			fi
-		else
-			return 1
-		fi
+pppd_is_ge_250()
+{
+	[ $(pppd_version_int) -ge 002005000 ]
+}
+
+pppd_pidfile()
+{
+	if pppd_is_ge_250; then
+		echo "/run/pppd/ppp-${IFACE}.pid"
 	else
-		return 1
+		echo "/run/ppp-${IFACE}.pid"
 	fi
 }
 
@@ -225,16 +223,18 @@ pppd_pre_start()
 	fi
 	[ "${insert_link_in_opts}" = "0" ] || opts="${link} ${opts}"
 
+	local pidfile="$(pppd_pidfile)"
+
 	ebegin "Starting pppd in ${IFACE}"
 	mark_service_inactive
 	if [ -n "${username}" ] \
 	&& [ -n "${password}" -o -z "${passwordset}" ]; then
 		printf "%s" "${password}" | \
 		eval start-stop-daemon --start --exec /usr/sbin/pppd \
-			--pidfile "/run/ppp-${IFACE}.pid" -- "${opts}" >/dev/null
+			--pidfile "${pidfile}" -- "${opts}" >/dev/null
 	else
 		eval start-stop-daemon --start --exec /usr/sbin/pppd \
-			--pidfile "/run/ppp-${IFACE}.pid" -- "${opts}" >/dev/null
+			--pidfile "${pidfile}" -- "${opts}" >/dev/null
 	fi
 
 	if ! eend $? "Failed to start PPP"; then
@@ -261,9 +261,13 @@ pppd_start()
 pppd_stop()
 {
 	yesno ${IN_BACKGROUND} && return 0
-	local pidfile="/run/ppp-${IFACE}.pid"
 
-	[ ! -s "${pidfile}" ] && return 0
+	local pidfile="$(pppd_pidfile)"
+	if [ ! -s "${pidfile}" ]; then
+		# Try the old path.
+		pidfile="/run/ppp-${IFACE}.pid"
+		[ -s "${pidfile}" ] || return 0
+	fi
 
 	# Give pppd at least 30 seconds do die, #147490
 	einfo "Stopping pppd on ${IFACE}"
